__all__ = ["eff", "snli", "squad", "reddit"]


from sota_extractor.scrapers.eff import eff
from sota_extractor.scrapers.snli import snli
from sota_extractor.scrapers.squad import squad
from sota_extractor.scrapers.reddit import reddit
